;===============================================
;= TWI Initialization                          =
;===============================================
.equ TWI_ADR_W		= 0b10011110  	; LM75 Schreib-Adresse: 1001 A2 A1 A0 1=Read/0=Write
.equ TWI_ADR_R		= 0b10011111  	; LM75 Lese-Adresse: 1001 A2 A1 A0 1=Read/0=Write

.equ Start			= 0x08 ;Diese Werte werden im Programm mit den Rckgabewerten des Statusregisters (TWSR) verglichen
.equ MT_SLA_ACK		= 0x18 ;wenn Wenn Werte nicht bereinstimmen, dann war die bertragung nicht korrekt, es wird ein Fehlertext ausgegeben
.equ MT_DATA_ACK	= 0x28
.equ MR_SLA_ACK		= 0x40
.equ MR_DATA_ACK	= 0x50
.equ MR_DATA_NAK	= 0x58

TWI_INIT:					;Init fr 27,1.6kHz           ;F_SCL = F_CPU/(16+2TWBR*4^TWPS)
	ldi r16, 32
	out TWBR, r16			;Bit Rate Register laden
	ldi r16, (0<<TWPS1)|(0<<TWPS0) 
	out TWSR, r16			;Prescaler laden
	rcall rs_i2c_ini		;Debug Ausgabe
ret
;===============================================
;= Start senden                                =
;===============================================
TWI_START:	
	ldi r16, (1<<TWINT)|(1<<TWSTA)|(1<<TWEN)
	out TWCR, r16			;sende Startkondition
	rcall rs_i2c_start		;Debug Textausgabe
	rcall TWI_wait
;-----------------------------------------------
	in Status, TWSR
	andi Status, 0xf8
	rcall rs_i2c_status		;Debug Textausgabe
	rcall rs_i2c_daten      ;Debug Wertausgabe (Statusbyte)
	cpi Status, Start
;	brne Fehler
ret
;===============================================
;= Slave Write Adresse bermitteln             =
;===============================================
TWI_SlaveWriteAdresse:
	ldi r16, TWI_ADR_W		;LM75 Schreib-Adresse
	out TWDR, r16			
	ldi r16, (1<<TWINT)|(1<<TWEN)	
	out TWCR, r16			;starte bermittlung
	rcall rs_i2c_slave		;Debug Textausgabe
	rcall TWI_wait	
;-----------------------------------------------
	in Status, TWSR
	andi Status, 0xf8
	rcall rs_i2c_status		;Debug Textausgabe
	rcall rs_i2c_daten      ;Debug Wertausgabe (Statusbyte)
	cpi Status, MT_SLA_ACK
	brne Fehler
ret
;===============================================
;= Slave Read Adresse bermitteln              =
;===============================================
TWI_SlaveReadAdresse:
	ldi r16, TWI_ADR_R		;LM75 Lese-Adresse
	out TWDR, r16			
	ldi r16, (1<<TWINT)|(1<<TWEN)	
	out TWCR, r16			;starte bermittlung
	rcall rs_i2c_slave		;Debug Ausgabe
	rcall TWI_wait	
;-----------------------------------------------
	in Status, TWSR
	andi Status, 0xf8
	rcall rs_i2c_status		;Debug Textausgabe
	rcall rs_i2c_daten      ;Debug Wertausgabe (Statusbyte)
	cpi Status, MR_SLA_ACK
	brne Fehler
ret
;===============================================
;= Daten bermitteln                           =
;===============================================
TWI_Daten_schreiben:
	ldi r16, 0x00			;Pointer-Register $00
	out TWDR, r16
	ldi r16, (1<<TWINT)|(1<<TWEN)	
	out TWCR, r16			;starte bermittlung
	rcall rs_i2c_senden		;Debug Textausgabe
	rcall TWI_wait
;-----------------------------------------------
	in Status, TWSR
	andi Status, 0xf8
	rcall rs_i2c_status		;Debug Textausgabe
	rcall rs_i2c_daten      ;Debug Wertausgabe (Statusbyte)
	cpi Status, MT_DATA_ACK
;	brne Fehler
ret
;===============================================
;= Daten empfangen                             =
;===============================================
TWI_Daten_lesen_MSB:
	ldi r16, (1<<TWINT)|(1<<TWEN)|(1<<TWEA)
	out TWCR, r16			;starte bermittlung
	rcall TWI_wait
;-----------------------------------------------
	in Status, TWDR
	rcall rs_i2c_empfang	;Info Textausgabe
	rcall rs_i2c_daten		;Info Wertausgabe (Datenbyte)
	in Status, TWSR
	andi Status, 0xf8
	rcall rs_i2c_status		;Debug Textausgabe
	rcall rs_i2c_daten      ;Debug Wertausgabe (Statusbyte)
ret

TWI_Daten_lesen_LSB:
	ldi r16, (1<<TWINT)|(1<<TWEN)	
	out TWCR, r16			;starte bermittlung
	rcall TWI_wait
;-----------------------------------------------
	in Status, TWDR
	rcall rs_i2c_empfang	;Info Textausgabe
	rcall rs_i2c_daten		;Info Wertausgabe (Datenbyte)
	in Status, TWSR
	andi Status, 0xf8
	rcall rs_i2c_status		;Debug Textausgabe
	rcall rs_i2c_daten      ;Debug Wertausgabe (Statusbyte)
ret

;===============================================
;= Stop senden                                 =
;===============================================
TWI_STOP:
	ldi r16, (1<<TWINT)|(1<<TWSTO)|(1<<TWEN)
	out TWCR, r16			;sende Startkondition
	rcall rs_i2c_stop		;Debug Textausgabe
ret
;===============================================
;= TWINT Flag abwarten                         =
;===============================================
TWI_wait:
	in r16, TWCR
	sbrs r16, TWINT
	rjmp TWI_wait
ret
;===============================================
;= Fehler                                      =
;===============================================
Fehler:
	rcall rs_i2c_fehler		;Debug Textausgabe
rjmp main
